import unittest
import numpy as np
from tests_configuration import *


class TestTripletsCSVFileFormat(unittest.TestCase):

    # 1) test file name format: must have the form '<dataset_name>_stimuli.csv'
    def test_csv_name_suffix(self):
        csv_name = CFG.TRIPLETS_PATH.split('/').pop()
        # At most one '_' in csv name, separating dataset name and stimuli
        self.assertEqual(len(csv_name.split('_')), 2)
        # must finish by '_stimuli.csv'
        self.assertEqual(csv_name.split('_').pop(), 'triplets.csv')

    # 2) Test csv name prefix (same as the folder)
    def test_csv_name_prefix(self):
        # Extract dataset name
        dataset_name = CFG.TRIPLETS_PATH.split('/')[-3]
        # Extract csv name
        csv_name = CFG.TRIPLETS_PATH.split('/').pop()
        csv_name_prefix = csv_name.split('_')[0]
        self.assertEqual(dataset_name, csv_name_prefix)


    # 3) test keys list
    def test_key_contents(self):
        for key in CFG.STIMULI_DATA.keys():
            if key.startswith('Unnamed'):
                continue
            self.assertIn(key, ['triplet_id', 'TGT_item', 'OTH_item',
                                'X_item', 'speaker_TGT', 'speaker_OTH',
                                'speaker_X', 'language_TGT', 'language_OTH',
                                'language_X', 'TGT_first'])

    # 4) test each key format
    def test_triplet_id_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertTrue(CFG.TRIPLETS_DATA['triplet_id'][i].startswith('triplet_'))

    # 4) e) test TGT language format (str, capital letters)
    def test_language_TGT_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['language_TGT'][i], str)
            self.assertTrue(CFG.TRIPLETS_DATA['language_TGT'][i].isupper())

    # 4) e) test OTH language format (str, capital letters)
    def test_language_OTH_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['language_OTH'][i], str)
            self.assertTrue(CFG.TRIPLETS_DATA['language_OTH'][i].isupper())

    # 4) e) test X language format (str, capital letters)
    def test_language_X_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['language_X'][i], str)
            self.assertTrue(CFG.TRIPLETS_DATA['language_X'][i].isupper())

    # 4) e) test speaker TGT format (str, capital letters)
    def test_speaker_TGT_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['speaker_TGT'][i], str)

    # 4) e) test speaker OTH format (str, capital letters)
    def test_speaker_OTH_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['speaker_OTH'][i], str)

    # 4) e) test speaker X format (str, capital letters)
    def test_speaker_X_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['speaker_X'][i], str)

    # 4) e) test TGT item format (str, capital letters)
    def test_TGT_item_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['TGT_item'][i], str)
            self.assertTrue(CFG.TRIPLETS_DATA['TGT_item'][i].endswith('.wav'))

    # 4) e) test TGT item format (str, capital letters)
    def test_OTH_item_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['OTH_item'][i], str)
            self.assertTrue(CFG.TRIPLETS_DATA['OTH_item'][i].endswith('.wav'))

    # 4) e) test TGT item format (str, capital letters)
    def test_X_item_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['X_item'][i], str)
            self.assertTrue(CFG.TRIPLETS_DATA['X_item'][i].endswith('.wav'))

    # 4) e) test TGT item format (str, capital letters)
    def test_TGT_first_format(self):
        random_indexes = np.random.randint(0, CFG.NB_TRIPLETS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.TRIPLETS_DATA['TGT_first'][i], bool)


if __name__ == '__main__':
    unittest.main()
